import React, {Component} from 'react';
import PropTypes from 'prop-types';

const style = {
  card: {
    fontFamily: 'Roboto, sans-serif',
    flexGrow: 1,
    height: 215,
    margin: '8px 0',
    position: 'relative',
    boxShadow: '0 3px 6px rgba(0, 0, 0, 0.16), 0 3px 6px rgba(0, 0, 0, 0.23)'
  },
  cardHover: {
    boxShadow: '0 14px 28px rgba(0, 0, 0, 0.25), 0 10px 10px rgba(0, 0, 0, 0.22)'
  },
  container: {
    padding: 12,
    float: 'left'
  },
  containerWithImage: {
    position: 'absolute',
    left: 184
  },
  headerButton: {
    position: 'absolute',
    right: 8,
    top: 8,
    cursor: 'pointer'
  },
  image: {
    height: 164,
    width: 184,
    float: 'left',
    backgroundSize: 'cover'
  },
  header: {
    padding: '0 0 8px'
  },
  headerTitle: {
    color: '#263238',
    margin: 0,
    textTransform: 'uppercase',
    fontSize: '.9rem'
  },
  headerSubtitle: {
    color: '#90A4AE',
    margin: 0,
    fontSize: '.8rem'
  },
  headerText: {
    color: '#607D8B',
    margin: 0,
    fontSize: '.9rem'
  },
  contentTitle: {
    margin: 0
  },
  contentText: {
    color: '#607D8B',
    margin: '2px 0 0',
    fontSize: '.9rem'
  },
  contentPrimary: {
    padding: '8px 0 0',
    color: '#263238',
    fontSize: '.9rem'
  },
  contentSecondary: {
    padding: '6px 16px 0 0',
    color: '#263238',
    fontSize: '.9rem'
  },
  footer: {
    position: 'absolute',
    backgroundColor: '#E3E2E2',
    width: '100%',
    height: 51,
    top: 'auto',
    right: 0,
    bottom: 0,
    left: 0
  },
  footerHover: {
    cursor: 'pointer',
    backgroundColor: '#DBDBDB'
  },
  footerStatus: {
    lineHeight: '12px',
    float: 'left'
  },
  footerStatusIcon: {
    lineHeight: '12px',
    padding: '14px 0 14px 12px',
    float: 'left'
  },
  footerStatusName: {
    lineHeight: '51px',
    float: 'left',
    paddingLeft: 12
  },
  footerButton: {
    float: 'right',
    lineHeight: '51px',
    borderLeft: '1px solid #CCCBCB',
    padding: '0 12px',
    textTransform: 'uppercase',
    fontSize: '.9rem'
  }
};

class CompatCard extends Component {
  constructor(props) {
    super(props);
    this.state = {
      cardHovered: false,
      footerHovered: false
    };
    this.colorForStatus = new Map([
       ['awaiting', {color: '#BF360C'}],
       ['checking', {color: '#FF6F00'}],
       ['finished', {color: '#43A047'}],
       ['running', {color: '#1976D2'}]
    ]);
  }

  handleToggleFooterHover = () => {
    const {footerHovered} = this.state;
    this.setState({footerHovered: !footerHovered});
  }

  handleToggleCardHover = () => {
    const {cardHovered} = this.state;
    this.setState({cardHovered: !cardHovered});
  }

  handleSelect = () => {
    const {onSelect} = this.props;
    onSelect && onSelect(this.props.id);
  }

  handleFooterClick = () => {
    const {onFooterButtonClick} = this.props;
    onFooterButtonClick && onFooterButtonClick(this.props.id);
  }

  handleHeaderClick = () => {
    const {onHeaderButtonClick} = this.props;
    onHeaderButtonClick && onHeaderButtonClick(this.props.id);
  }

  renderHeader = () => {
    const {title, subtitle, headerText} = this.props;
    return (
      <header style={style.header}>
        <h3 style={style.headerTitle}>{title}</h3>
        <p style={style.headerSubtitle}>{subtitle}</p>
        <p style={style.headerText}>{headerText}</p>
      </header>
    );
  };

  renderContent = () => {
    const {
      contentTitle,
      contentText,
      primaryText,
      secondaryText
    } = this.props;

    return (
      <main className="content">
        <p style={style.contentTitle}>{contentTitle}</p>
        <p style={style.contentText}>{contentText}</p>
        <div style={style.contentPrimary}>{primaryText}</div>
        <div style={style.contentSecondary}>{secondaryText}</div>
      </main>
    );
  };

  renderFooterIconAndText = () => {
    const {footerIcon, footerStatus} = this.props;
    const {status, text} = footerStatus;
    const {color} = this.colorForStatus.get(status);

    return (
      <div
        style={{...style.footerStatus, color}}
      >
        {!!footerIcon &&
          <span style={style.footerStatusIcon}>
            {footerIcon}
          </span>
        }
        <strong style={style.footerStatusName}>{text}</strong>
      </div>
    );
  };

  renderFooter = () => {
    const {onFooterButtonClick, footerButtonLabel} = this.props;
    const hasButton = !!onFooterButtonClick;

    const footerSyle = this.state.footerHovered ?
      {...style.footer, ...style.footerHover} :
      style.footer;

    return (
      <footer
        className="card__footer"
        style={footerSyle}
        onMouseEnter={this.handleToggleFooterHover}
        onMouseLeave={this.handleToggleFooterHover}
        onClick={this.handleFooterClick}
      >
        {!!this.props.footerStatus && this.renderFooterIconAndText()}
        {hasButton &&
          <div style={style.footerButton}>
            {footerButtonLabel}
          </div>
        }
      </footer>
    );
  };

  render() {
    const {image, onSelect, headerButton} = this.props;
    const hasImage = !!image;
    const canSelect = !!onSelect;
    const hasHeaderButton = !!headerButton;

    const boxShadow = this.state.cardHovered ?
      style.cardHover.boxShadow :
      style.card.boxShadow;

    const cursor = canSelect ? 'pointer': 'default';

    const containerWithImage = hasImage ? style.containerWithImage : {}

    const containerStyle = {
      ...style.container,
      ...containerWithImage,
      cursor
    };

    return (
      <div
        style={{...style.card, boxShadow}}
        onMouseEnter={this.handleToggleCardHover}
        onMouseLeave={this.handleToggleCardHover}
      >
        {hasImage &&
          <div
            style={{...style.image, backgroundImage: `url(${this.props.image})`}}
          />
        }
        {hasHeaderButton &&
          <div
            className="card__header_button"
            style={style.headerButton}
            onClick={this.handleHeaderClick}
          >
            {headerButton}
          </div>
        }
        <div
          className="card__container"
          style={containerStyle}
          onClick={this.handleSelect}
        >
          {this.renderHeader()}
          {this.renderContent()}
        </div>
        {this.renderFooter()}
      </div>
    );
  }
}

CompatCard.propTypes = {
  id: PropTypes.string.isRequired,
  title: PropTypes.string.isRequired,
  image: PropTypes.string,
  subtitle: PropTypes.string,
  headerText: PropTypes.string,
  contentTitle: PropTypes.string,
  contentText: PropTypes.string,
  primaryText: PropTypes.string,
  secondaryText: PropTypes.string,
  headerButton: PropTypes.node,
  footerButtonLabel: PropTypes.string,
  footerIcon: PropTypes.node,
  footerStatus: PropTypes.shape({
    status: PropTypes.string,
    text: PropTypes.string
  }),
  onSelect: PropTypes.func,
  onFooterButtonClick: PropTypes.func,
  onHeaderButtonClick: PropTypes.func
};

export default CompatCard;
